/*
*******************************************************************************
**
**  This device driver was created by Applilet2 for 78K0/Kx2
**  8-Bit Single-Chip Microcontrollers
**
**  Copyright(C) NEC Electronics Corporation 2002 - 2011
**  All rights reserved by NEC Electronics Corporation.
**
**  This program should be used on your own responsibility.
**  NEC Electronics Corporation assumes no responsibility for any losses
**  incurred by customers or third parties arising from the use of this file.
**
**  Filename :	Port.c
**  Abstract :	This file implements device driver for PORT module.
**  APIlib :	Applilet2 for 78K0/Kx2 V2.41 [29 Apr. 2008]
**
**  Device :	uPD78F0512_48
**
**  Compiler :	CC78K0
**
**  Creation date:	2011/12/24
**  
*******************************************************************************
*/

/*
*******************************************************************************
**  Include files
*******************************************************************************
*/
#include "macrodriver.h"
#include "Port.h"
/* Start user code for include definition. Do not edit comment generated here */
/* End user code for include definition. Do not edit comment generated here */
#include "user_define.h"

/*
*******************************************************************************
**  Global define
*******************************************************************************
*/
/* Start user code for global definition. Do not edit comment generated here */
/* End user code for global definition. Do not edit comment generated here */



/*
**-----------------------------------------------------------------------------
**
**	Abstract:
**		This function initializes setting for Port I/O.
**
**	Parameters:
**		None
**
**	Returns:
**		None
**
**-----------------------------------------------------------------------------
*/
void PORT_Init( void )
{
	P0 = Pn0_OUTPUT_0 | Pn1_OUTPUT_0;
	P1 = Pn5_OUTPUT_0 | Pn7_OUTPUT_0;
	P2 = Pn0_OUTPUT_0 | Pn1_OUTPUT_0 | Pn2_OUTPUT_0 | Pn3_OUTPUT_0 | Pn4_OUTPUT_0 | Pn5_OUTPUT_0 | Pn6_OUTPUT_0;
	P3 = Pn1_OUTPUT_0 | Pn2_OUTPUT_0 | Pn3_OUTPUT_0;
	P4 = Pn0_OUTPUT_0 | Pn1_OUTPUT_0;
	P6 = Pn2_OUTPUT_0 | Pn3_OUTPUT_0;
	P7 = Pn0_OUTPUT_0 | Pn1_OUTPUT_0 | Pn2_OUTPUT_0 | Pn3_OUTPUT_0 | Pn4_OUTPUT_0 | Pn5_OUTPUT_0;
	P12 = Pn0_OUTPUT_0 | Pn1_OUTPUT_0 | Pn2_OUTPUT_0 | Pn3_OUTPUT_0 | Pn4_OUTPUT_0;
	P13 = Pn0_OUTPUT_0;
	P14 = Pn0_OUTPUT_0;

	PU1 = PUn0_PULLUP_ON | PUn1_PULLUP_ON | PUn2_PULLUP_ON | PUn3_PULLUP_ON | PUn4_PULLUP_ON | PUn6_PULLUP_ON;
	PU3 = PUn0_PULLUP_ON;
	PM0 = PMn0_MODE_OUTPUT | PMn1_MODE_OUTPUT | PM0_DEFAULT;
	PM1 = PMn0_MODE_INPUT | PMn1_MODE_INPUT | PMn2_MODE_INPUT | PMn3_MODE_INPUT | PMn4_MODE_INPUT | PMn5_MODE_OUTPUT | PMn6_NOT_USE | PMn7_MODE_OUTPUT;
	PM2 = PMn0_MODE_OUTPUT | PMn1_MODE_OUTPUT | PMn2_MODE_OUTPUT | PMn3_MODE_OUTPUT | PMn4_MODE_OUTPUT | PMn5_MODE_OUTPUT | PMn6_MODE_OUTPUT | PMn7_NOT_USE;
	PM3 = PMn0_NOT_USE | PMn1_MODE_OUTPUT | PMn2_MODE_OUTPUT | PMn3_MODE_OUTPUT | PM3_DEFAULT;
	PM4 = PMn0_MODE_OUTPUT | PMn1_MODE_OUTPUT | PM4_DEFAULT;
	PM6 = PMn0_NOT_USE | PMn1_NOT_USE | PMn2_MODE_OUTPUT | PMn3_MODE_OUTPUT | PM6_DEFAULT;
	PM7 = PMn0_MODE_OUTPUT | PMn1_MODE_OUTPUT | PMn2_MODE_OUTPUT | PMn3_MODE_OUTPUT | PMn4_MODE_OUTPUT | PMn5_MODE_OUTPUT | PM7_DEFAULT;
	PM12 = PMn0_MODE_OUTPUT | PMn1_MODE_OUTPUT | PMn2_MODE_OUTPUT | PMn3_MODE_OUTPUT | PMn4_MODE_OUTPUT | PM12_DEFAULT;
	PM14 = PMn0_MODE_OUTPUT | PM14_DEFAULT;
	ADPC = PORT_ADPC_7DIO;
}

/*
**-----------------------------------------------------------------------------
**
**	Abstract:
**		This function changes P15 to input mode by software.
**
**	Parameters:
**		enablePU :	Pull-up enable or not
**
**	Returns:
**		None
**
**-----------------------------------------------------------------------------
*/
void PORT_ChangeP15Input( BOOL enablePU )
{
	if( enablePU )
	{
		PU1 |= PUn5_PULLUP_ON;
	}
	else
	{
		PU1 &= (UCHAR)~PUn5_PULLUP_ON;
	}
	
	PM1 |= PMn5_MODE_INPUT;
}

/*
**-----------------------------------------------------------------------------
**
**	Abstract:
**		This function changes P15 to output mode by software.
**
**	Parameters:
**		initialValue :	initial output 1 or 0
**
**	Returns:
**		None
**
**-----------------------------------------------------------------------------
*/
void PORT_ChangeP15Output( BOOL initialValue )
{
	if( initialValue )
	{
		P1 |= Pn5_OUTPUT_1;
	}
	else
	{
		P1 &= (UCHAR)~Pn5_OUTPUT_1;
	}
	PM1 &= PMn5_CHANGE_OUTPUT;
}

/*
**-----------------------------------------------------------------------------
**
**	Abstract:
**		This function changes P17 to input mode by software.
**
**	Parameters:
**		enablePU :	Pull-up enable or not
**
**	Returns:
**		None
**
**-----------------------------------------------------------------------------
*/
void PORT_ChangeP17Input( BOOL enablePU )
{
	if( enablePU )
	{
		PU1 |= PUn7_PULLUP_ON;
	}
	else
	{
		PU1 &= (UCHAR)~PUn7_PULLUP_ON;
	}
	
	PM1 |= PMn7_MODE_INPUT;
}

/*
**-----------------------------------------------------------------------------
**
**	Abstract:
**		This function changes P17 to output mode by software.
**
**	Parameters:
**		initialValue :	initial output 1 or 0
**
**	Returns:
**		None
**
**-----------------------------------------------------------------------------
*/
void PORT_ChangeP17Output( BOOL initialValue )
{
	if( initialValue )
	{
		P1 |= Pn7_OUTPUT_1;
	}
	else
	{
		P1 &= (UCHAR)~Pn7_OUTPUT_1;
	}
	PM1 &= PMn7_CHANGE_OUTPUT;
}

/*
**-----------------------------------------------------------------------------
**
**	Abstract:
**		This function changes P33 to input mode by software.
**
**	Parameters:
**		enablePU :	Pull-up enable or not
**
**	Returns:
**		None
**
**-----------------------------------------------------------------------------
*/
void PORT_ChangeP33Input( BOOL enablePU )
{
	if( enablePU )
	{
		PU3 |= PUn3_PULLUP_ON;
	}
	else
	{
		PU3 &= (UCHAR)~PUn3_PULLUP_ON;
	}
	
	PM3 |= PMn3_MODE_INPUT;
}

/*
**-----------------------------------------------------------------------------
**
**	Abstract:
**		This function changes P33 to output mode by software.
**
**	Parameters:
**		initialValue :	initial output 1 or 0
**
**	Returns:
**		None
**
**-----------------------------------------------------------------------------
*/
void PORT_ChangeP33Output( BOOL initialValue )
{
	if( initialValue )
	{
		P3 |= Pn3_OUTPUT_1;
	}
	else
	{
		P3 &= (UCHAR)~Pn3_OUTPUT_1;
	}
	PM3 &= PMn3_CHANGE_OUTPUT;
}

/* Start adding user code. Do not edit comment generated here */
/* End user code adding. Do not edit comment generated here */


