/*
*******************************************************************************
**
**  This device driver was created by Applilet2 for 78K0/Kx2
**  8-Bit Single-Chip Microcontrollers
**
**  Copyright(C) NEC Electronics Corporation 2002 - 2011
**  All rights reserved by NEC Electronics Corporation.
**
**  This program should be used on your own responsibility.
**  NEC Electronics Corporation assumes no responsibility for any losses
**  incurred by customers or third parties arising from the use of this file.
**
**  Filename :	 main.c
**  Abstract :	This file implements main function.
**  APIlib :	Applilet2 for 78K0/Kx2 V2.41 [29 Apr. 2008]
**
**  Device :	uPD78F0512_48
**
**  Compiler :	CC78K0
**
**  Creation date:	2011/12/23
**  
*******************************************************************************
*/

/*
*******************************************************************************
**  Include files
*******************************************************************************
*/
#include "macrodriver.h"
#include "Int.h"
#include "Ad.h"
#include "Serial.h"
#include "Timer.h"
#include "Port.h"
#include "System.h"
/* Start user code for include definition. Do not edit comment generated here */
/* End user code for include definition. Do not edit comment generated here */
#include "user_define.h"

/*
*******************************************************************************
**  Global define
*******************************************************************************
*/
/* Start user code for global definition. Do not edit comment generated here */
/* End user code for global definition. Do not edit comment generated here */
/*
**-----------------------------------------------------------------------------
**
**	Abstract:
**		This function  implements main function.
**
**	Parameters:
**		None
**
**	Returns:
**		None
**
**-----------------------------------------------------------------------------
*/
#define u32 unsigned long
#define u16 unsigned int
#define u8  unsigned char
#define i8  char

/*
//==================================================================
// General Commands
//==================================================================
*/

/* STATUS bits - Used by all methods */
#define STCINT  0x01
#define ASQINT  0x02
#define RDSINT  0x04
#define RSQINT  0x08
#define ERR     0x40
#define CTS     0x80

/* POWER_UP*/
#define POWER_UP                      0x01
#define POWER_UP_IN_FUNC_FMRX         0x00
#define POWER_UP_IN_FUNC_AMRX         0x01
#define POWER_UP_IN_FUNC_FMTX         0x02
#define POWER_UP_IN_FUNC_WBRX         0x03
#define POWER_UP_IN_FUNC_QUERY        0x0F
#define POWER_UP_IN_PATCH             0x20
#define POWER_UP_IN_GPO2OEN           0x40
#define POWER_UP_IN_CTSIEN            0x80
#define POWER_UP_IN_OPMODE_RX_ANALOG  0x05
#define POWER_UP_IN_OPMODE_TX_ANALOG  0x50
#define POWER_UP_IN_XOSCEN			  0x10

/* GET_REV */
#define GET_REV 0x10

/* POWER_DOWN */
#define POWER_DOWN 0x11

/* SET_PROPERTY */
#define SET_PROPERTY 0x12

/* GET_PROPERTY */
#define GET_PROPERTY 0x13

/* GET_INT_STATUS */
#define GET_INT_STATUS 0x14

/*
//==================================================================
// AM Receive Commands
//==================================================================
*/
/* AM_TUNE_FREQ */
#define AM_TUNE_FREQ 0x40

/* AM_TUNE_STATUS */
#define AM_TUNE_STATUS           0x42
#define AM_TUNE_STATUS_IN_INTACK 0x01
#define AM_TUNE_STATUS_IN_CANCEL 0x02
#define AM_TUNE_STATUS_OUT_VALID 0x01
#define AM_TUNE_STATUS_OUT_AFCRL 0x02
#define AM_TUNE_STATUS_OUT_BTLF  0x80

#define	AM_RSQ_STATUS			0x43
/*
//==================================================================
// AM Receive Properties
//==================================================================
*/
/* AM_DEEMPHASIS */
#define AM_DEEMPHASIS      0x3100
#define AM_DEEMPHASIS_MASK 0x0001
#define AM_DEEMPHASIS_SHFT 0

/* AM_CHANNEL_FILTER*/
#define AM_CHANNEL_FILTER      0x3102
#define AM_CHANNEL_FILTER_MASK 0x0007
#define AM_CHANNEL_FILTER_SHFT 0

/* AM_SOFT_MUTE_RATE */
#define AM_SOFT_MUTE_RATE 0x3300

/* AM_SOFT_MUTE_SLOPE */
#define AM_SOFT_MUTE_SLOPE 0x3301
#define AM_SOFT_MUTE_SLOPE_MASK 0x000F
#define AM_SOFT_MUTE_SLOPE_SHFT 0

/* AM_SOFT_MUTE_MAX_ATTENUATION */
#define AM_SOFT_MUTE_MAX_ATTENUATION 0x3302
#define AM_SOFT_MUTE_MAX_ATTENUATION_MASK 0x003F
#define AM_SOFT_MUTE_MAX_ATTENUATION_SHFT 0

/* AM_SOFT_MUTE_SNR_THRESHOLD */
#define AM_SOFT_MUTE_SNR_THRESHOLD 0x3303
#define AM_SOFT_MUTE_SNR_THRESHOLD_MASK 0x003F
#define AM_SOFT_MUTE_SNR_THRESHOLD_SHFT 0

/* AM_SEEK_BAND_BOTTOM */
#define AM_SEEK_BAND_BOTTOM 0x3400

/* AM_SEEK_BAND_TOP */
#define AM_SEEK_BAND_TOP 0x3401

/* AM_SEEK_FREQ_SPACING */
#define AM_SEEK_FREQ_SPACING      0x3402
#define AM_SEEK_FREQ_SPACING_MASK 0x000F
#define AM_SEEK_FREQ_SPACING_SHFT 0

/* AM_SEEK_TUNE_SNR_THRESHOLD */
#define AM_SEEK_TUNE_SNR_THRESHOLD      0x3403
#define AM_SEEK_TUNE_SNR_THRESHOLD_MASK 0x003F
#define AM_SEEK_TUNE_SNR_THRESHOLD_SHFT 0

/* AM_SEEK_TUNE_RSSI_THRESHOLD */
#define AM_SEEK_TUNE_RSSI_THRESHOLD      0x3404
#define AM_SEEK_TUNE_RSSI_THRESHOLD_MASK 0x003F
#define AM_SEEK_TUNE_RSSI_THRESHOLD_SHFT 0

/*
//==================================================================
// General Receive Properties
//==================================================================
*/
/* RX_VOLUME */
#define RX_VOLUME      0x4000
#define RX_VOLUME_MASK 0x003F
#define RX_VOLUME_SHFT 0

/* RX_HARD_MUTE */
#define RX_HARD_MUTE 0x4001
#define RX_HARD_MUTE_RMUTE_MASK 0x0001
#define RX_HARD_MUTE_LMUTE_MASK 0x0002
#define RX_HARD_MUTE_RMUTE_SHFT 0
#define RX_HARD_MUTE_LMUTE_SHFT 1
/*
//==================================================================
// Bit Definitions for Properties
//==================================================================

/* AM_CHANNEL_FILTER_BW */
#define AM_CHANNEL_FILTER_BW_6KHZ 0
#define AM_CHANNEL_FILTER_BW_4KHZ 1
#define AM_CHANNEL_FILTER_BW_3KHZ 2
#define AM_CHANNEL_FILTER_BW_2KHZ 3
/*
//-----------------------------------------------------------------------------
// Defines
//-----------------------------------------------------------------------------
*/
#define READ    1
#define WRITE   0

#define POWERUP_TIME 110    /* Powerup delay in milliseconds*/
#define GPO_IEN				 0x0001
#define GPO_IEN_STCIEN_MASK  0x0001


/*
//==================================================================
//
//==================================================================
*/
#define	SCLK				P3.2
#define	SDIO				P3.3
#define	SENB				P3.1
#define	RSTB				P4.0

#define	LCD_CMD_WAIT		400
#define	LCD_BIT_WAIT		10
#define	PRESET_MAX			20
#define	VOL_COUNT_SET		10

#define	FREQ_MIN			522
#define	FREQ_MAX			1620

#define FREQ_MANUAL_DIV		8

#define	LCD_DB4				P2.0
#define	LCD_DB5				P2.1
#define	LCD_DB6				P2.2
#define	LCD_DB7				P2.3
#define	LCD_RS				P13.0
#define	LCD_E				P14.0
#define	TUNE_OUT			P4.1

#define	POWER_CONT			P12.2
#define	POWER_KEY			P1.3
#define UP_KEY				P1.0
#define	DOWN_KEY			P1.1
#define	SELECT_KEY			P1.2

#define KYOKU_INIT			1000
#define KYOKU_ON			990
#define KYOKU_NEXT			500
#define	KYOKU_ON_2			200
#define RSQ_INIT			100

unsigned char	timer00_over;
u8  			cmd[8];
u8  			rsp[15];
u8  			RSSI;
u8  			STC;
u8  			BLTF;
u8  			AFCRL;
u8  			Valid;
u16 			Freq;
u8  			ASNR;
u16 			AntCap;

u8				partNumber;
char 			fwMajor;
char 			fwMinor;
u16 			patchID;
char			cmpMajor;
char			cmpMinor;
char			chipRev;

unsigned short	ad_lev_in;
unsigned short	vol_level;
unsigned short	vol_level_m;

unsigned int	khz_no;
unsigned int	khz_no_m;

unsigned char	mode;

unsigned int	khz_no_setmode;


unsigned char	preset_no;
unsigned int	preset[PRESET_MAX];
unsigned char	preset_no_setmode;
unsigned char	preset_ch[(FREQ_MAX-FREQ_MIN)/9+5];

unsigned char	set_count;
unsigned char	tun_flg;

unsigned int	kyoku_count;

unsigned char	count;
unsigned char	rec[12];
unsigned char	count10_flg;
unsigned char	receive_err;
unsigned char	receive_no;

unsigned char	i2c_radio_addr=0x22;
unsigned char	i2c_finish;

unsigned char	eeprom_addr=0xa0;
unsigned char	eeprom_write_data[3];
unsigned char	eeprom_read_data[1];

unsigned char	rsq_count;
unsigned char	bank_no;

struct {
	unsigned int PoweredUp : 1;
}Bit;


void	wait_time(unsigned int time)
{
	while(time!=0){
		time--;
	}
}

/*----------I2C,EEPROM ---------*/
void	i2c_radio_write(unsigned char number_bytes,unsigned char *data)
{
	i2c_finish = 0 ;
	IIC0_MasterSendStart(i2c_radio_addr, data, number_bytes,100);
	while (!i2c_finish){};
}
void	i2c_radio_read(unsigned char number_bytes,unsigned char *data)
{
	i2c_finish = 0 ;
	IIC0_MasterReceiveStart(i2c_radio_addr, data,number_bytes,100);
	while (!i2c_finish) {};
}
void	wait_i2c(void)
{
		wait_time(500);
}
void	eeprom_write(unsigned char ad1,unsigned char ad2,unsigned char d)
{
	eeprom_write_data[0]=ad1;
	eeprom_write_data[1]=ad2;
	eeprom_write_data[2]=d;
	i2c_finish = 0 ;
	IIC0_MasterSendStart(eeprom_addr, eeprom_write_data, 3,1);
	while (!i2c_finish){};
}
unsigned char	eeprom_read(unsigned char ad1,unsigned char ad2)
{
unsigned char	data;

	eeprom_write_data[0]=ad1;
	eeprom_write_data[1]=ad2;
	i2c_finish = 0 ;
	IIC0_MasterSendStart(eeprom_addr, eeprom_write_data, 2,1);
	while (!i2c_finish){};

	i2c_finish = 0 ;
	IIC0_MasterReceiveStart(eeprom_addr, eeprom_read_data,1,10);
	while (!i2c_finish) {};
	
	data=eeprom_read_data[0];
	
	return data;
}
void	wait_eeprom(void)
{
		wait_time(1000);
}
void	eeprom_write_sub(void)
{
unsigned char	i;

	for(i=0;i<PRESET_MAX;i++){
		eeprom_write(bank_no*2,0x10+i*2,(unsigned char)(preset[i]%256));wait_eeprom();
		eeprom_write(bank_no*2,0x11+i*2,(unsigned char)(preset[i]/256));wait_eeprom();
	}
}

void	eeprom_init(void)
{
unsigned char	i;

		preset[0]=FREQ_MAX;

		for(i=1;i<PRESET_MAX;i++){
			preset[i]=0;
		}
		
		khz_no=FREQ_MIN;
		
		eeprom_write(0,0,0xa5);wait_eeprom();
		
		bank_no=0;eeprom_write_sub();
		eeprom_write(bank_no*2,0x0a,(unsigned char)(khz_no%256));wait_eeprom();
		eeprom_write(bank_no*2,0x0b,(unsigned char)(khz_no/256));wait_eeprom();
		eeprom_write(bank_no*2,0x08,preset_no);wait_eeprom();
		eeprom_write(bank_no*2,0x05,mode);wait_eeprom();
		
		bank_no=1;eeprom_write_sub();
		eeprom_write(bank_no*2,0x0a,(unsigned char)(khz_no%256));wait_eeprom();
		eeprom_write(bank_no*2,0x0b,(unsigned char)(khz_no/256));wait_eeprom();
		eeprom_write(bank_no*2,0x08,preset_no);wait_eeprom();
		eeprom_write(bank_no*2,0x05,mode);wait_eeprom();

		bank_no=2;eeprom_write_sub();
		eeprom_write(bank_no*2,0x0a,(unsigned char)(khz_no%256));wait_eeprom();
		eeprom_write(bank_no*2,0x0b,(unsigned char)(khz_no/256));wait_eeprom();
		eeprom_write(bank_no*2,0x08,preset_no);wait_eeprom();
		eeprom_write(bank_no*2,0x05,mode);wait_eeprom();

		bank_no=0;
		eeprom_write(0,0x0c,bank_no);wait_eeprom();
}
void	eeprom_read_preset(void)
{
unsigned char	i;
unsigned int sub0,sub1;

		for(i=0;i<PRESET_MAX;i++){
			sub0=(unsigned int)eeprom_read(bank_no*2,0x10+i*2);wait_eeprom();
			sub1=(unsigned int)eeprom_read(bank_no*2,0x11+i*2);wait_eeprom();
			preset[i]=sub0+sub1*256;
		}
}

void	eeprom_check(void)
{
unsigned int sub0,sub1;

	if(eeprom_read(0,0)==0xa5){
		bank_no=eeprom_read(0,0x0c);wait_eeprom();
		sub0=(unsigned int)eeprom_read(bank_no*2,0x0a);wait_eeprom();
		sub1=(unsigned int)eeprom_read(bank_no*2,0x0b);wait_eeprom();
		khz_no=sub0+sub1*256;
		preset_no=eeprom_read(bank_no*2,0x08);wait_eeprom();
		mode=eeprom_read(bank_no*2,0x05);wait_eeprom();
		eeprom_read_preset();
		
	}
	else{
		eeprom_init();
	}
}
void	write_direct(void){

		eeprom_write(0,0x0c,bank_no);wait_eeprom();
		eeprom_write(bank_no*2,0x0a,(unsigned char)(khz_no%256));wait_eeprom();
		eeprom_write(bank_no*2,0x0b,(unsigned char)(khz_no/256));wait_eeprom();
		eeprom_write(bank_no*2,0x08,preset_no);wait_eeprom();
		eeprom_write(bank_no*2,0x05,mode);wait_eeprom();
}
void	write_direct_set(unsigned char	no){

		eeprom_write(bank_no*2,0x10+no*2,(unsigned char)(preset[no]%256));wait_eeprom();
		eeprom_write(bank_no*2,0x11+no*2,(unsigned char)(preset[no]/256));wait_eeprom();
}

/*----------  LCD MODULE ----------*/
void	lcd_db(unsigned char data)
{
	LCD_DB4=data&0x01;
	LCD_DB5=(data&0x02)>>1;
	LCD_DB6=(data&0x04)>>2;
	LCD_DB7=(data&0x08)>>3;
}
void	lcd_wait(void)
{
	wait_time(5000);
}
void	lcd_c_set(void)
{
	LCD_RS=0;
	LCD_E=1;
	wait_time(LCD_BIT_WAIT);
	LCD_E=0;
}
void	lcd_d_set(void)
{
	LCD_RS=1;
	LCD_E=1;
	wait_time(LCD_BIT_WAIT);
	LCD_E=0;
}
void	lcd_cmd(unsigned char data)
{
	lcd_db((data&0xf0)>>4);
	lcd_c_set();
	lcd_db(data&0x0f);
	lcd_c_set();
	wait_time(LCD_CMD_WAIT);
}
void	lcd_data(unsigned char data)
{
	lcd_db((data&0xf0)>>4);
	lcd_d_set();
	lcd_db(data&0x0f);
	lcd_d_set();
	wait_time(LCD_CMD_WAIT);
}
void	lcd_init(void)
{
	lcd_wait();
	lcd_db(3);
	lcd_c_set();
	lcd_wait();
	lcd_db(3);
	lcd_c_set();
	lcd_wait();
	lcd_db(3);
	lcd_c_set();
	lcd_wait();
	lcd_db(2);
	lcd_c_set();
	lcd_wait();
	
	lcd_cmd(0x28);
	lcd_cmd(0x08);
	lcd_cmd(0x0c);
	lcd_cmd(0x06);
}
void	lcd_clear(void)
{
	lcd_cmd(0x01);
}
void	lcd_xy(unsigned char x,unsigned char y)
{
unsigned char data;

	data=0x80;
	switch (y){
		case 0:data=0x80;break;
		case 1:data=0xc0;break;
		case 2:data=0x90;break;
		case 3:data=0xd0;break;
	}
	data=data+x;
	lcd_cmd(data);
}
void	lcd_print(char *data)
{
	while (*data!=0){
		lcd_data(*data++);
	}
}
void	lcd_su(unsigned char data)
{
	switch (data){
		case 0:lcd_print("0");break;
		case 1:lcd_print("1");break;
		case 2:lcd_print("2");break;
		case 3:lcd_print("3");break;
		case 4:lcd_print("4");break;
		case 5:lcd_print("5");break;
		case 6:lcd_print("6");break;
		case 7:lcd_print("7");break;
		case 8:lcd_print("8");break;
		case 9:lcd_print("9");break;
		case 10:lcd_print("A");break;
		case 11:lcd_print("B");break;
		case 12:lcd_print("C");break;
		case 13:lcd_print("D");break;
		case 14:lcd_print("E");break;
		case 15:lcd_print("F");break;
	}
}
void	lcd_pr_byte(unsigned char data)
{
unsigned char data2;

	data2=data%16;data=data/16;
	lcd_su(data);
	lcd_su(data2);
}
void	lcd_ini_disp(void)
{
	lcd_xy(0,0);
	lcd_print("    kHz");
}

void	lcd_1keta10(unsigned char count)
{
unsigned char n0;

	n0=(unsigned char)(count%10);

	lcd_su(n0);
}

void	lcd_2keta10(unsigned char count)
{
unsigned char n0,n1;

	n0=(unsigned char)(count%10);count=(count-n0)/10;
	n1=(unsigned char)(count%10);

	lcd_su(n1);
	lcd_su(n0);
}
void	lcd_4keta10(unsigned int count)
{
unsigned char n0,n1,n2,n3;

	n0=(unsigned char)(count%10);count=(count-n0)/10;
	n1=(unsigned char)(count%10);count=(count-n1)/10;
	n2=(unsigned char)(count%10);count=(count-n2)/10;
	n3=(unsigned char)(count%10);

	lcd_su(n3);
	lcd_su(n2);
	lcd_su(n1);
	lcd_su(n0);
}
void	lcd_mode(unsigned char m)
{
	lcd_xy(0,1);
	switch(m){
		case 1:	lcd_print("TUNE");
				if(tun_flg==1){lcd_print("m");}else{lcd_print("a");}
				break;
		case 0:	lcd_print("CH-");lcd_2keta10(preset_no);break;
		case 2:	lcd_print("SET");lcd_2keta10(preset_no_setmode);break;
	}
}
void	lcd_khz(unsigned int hz)
{
unsigned char n0,n1,n2,n3;
unsigned int count;

	count=hz;
	n0=(unsigned char)(count%10);count=(count-n0)/10;
	n1=(unsigned char)(count%10);count=(count-n1)/10;
	n2=(unsigned char)(count%10);count=(count-n2)/10;
	n3=(unsigned char)(count%10);

	if(n3==0){lcd_print(" ");}
	else{lcd_su(n3);}
	lcd_su(n2);
	lcd_su(n1);
	lcd_su(n0);
}
void	lcd_ch_disp(void)
{
	if(preset_ch[(khz_no-FREQ_MIN)/9]!=0xff){
		lcd_xy(8,1);
		lcd_print("(CH-  )");
		lcd_xy(12,1);
		lcd_2keta10(preset_ch[(khz_no-FREQ_MIN)/9]);
	}
	else{
		lcd_xy(8,1);lcd_print("       ");
	}
}
void	lcd_ch_disp_sub(void)
{
	if(preset_ch[(khz_no-FREQ_MIN)/9]!=0xff){
		lcd_xy(8,1);
		lcd_print("(CH-  )");
		lcd_xy(12,1);
		lcd_2keta10(preset_ch[(khz_no-FREQ_MIN)/9]);
	}
}

void	lcd_kyokumei_clr(void)
{
	lcd_xy(5,1);
	lcd_print("           ");
}
void	lcd_kyokumei(unsigned int	no)
{
unsigned int	sub;
	sub=no/9;
	lcd_xy(8,1);
	switch(sub){
		case 62:lcd_print("KANSAI  ");break;
		case 63:lcd_print("NHK1    ");break;
		case 66:lcd_print("NHK1    ");break;
		case 68:lcd_print("NHK1    ");break;
		case 69:lcd_print("NHK1    ");break;
		case 74:lcd_print("NHK1    ");break;
		case 76:lcd_print("IBC/NHK1");break;
		case 77:lcd_print("NHK2    ");break;
		case 81:lcd_print("NHK1    ");break;
		case 82:lcd_print("KNB/RBC ");break;
		case 83:lcd_print("NHK2    ");break;
		case 84:lcd_print("NHK1    ");break;
		case 85:lcd_print("YBS/KRY ");break;
		case 86:lcd_print("NHK2    ");break;
		case 90:lcd_print("AFN     ");break;
		case 91:lcd_print("NHK1    ");break;
		case 92:lcd_print("NHK2    ");break;
		case 93:lcd_print("NHK1    ");break;
		case 94:lcd_print("NHK1    ");break;
		case 97:lcd_print("NHK2    ");break;
		case 99:lcd_print("NHK1    ");break;
		case 100:lcd_print("BSS/RKC ");break;
		case 102:lcd_print("YBC/KRY ");break;
		case 103:lcd_print("NHK1    ");break;
		case 104:lcd_print("ABS/MRT ");break;
		case 106:lcd_print("TBS     ");break;
		case 110:lcd_print("NHK1    ");break;
		case 112:lcd_print("ABC     ");break;
		case 113:lcd_print("NHK2    ");break;
		case 114:lcd_print("NHK1    ");break;
		case 117:lcd_print("CBC     ");break;
		case 121:lcd_print("NHK2    ");break;
		case 123:lcd_print("MBC/MRO ");break;
		case 124:lcd_print("BSN/RNB ");break;
		case 125:lcd_print("NHK2    ");break;
		case 126:lcd_print("BUNKA   ");break;
		case 127:lcd_print("KBS     ");break;
		case 131:lcd_print("MBS     ");break;
		case 132:lcd_print("NHK1    ");break;
		case 133:lcd_print("RKK/IBS ");break;
		case 136:lcd_print("NHK1    ");break;
		case 137:lcd_print("RAB/NBC ");break;
		case 138:lcd_print("NIPPON  ");break;
		case 140:lcd_print("TBC     ");break;
		case 141:lcd_print("JRT     ");break;
		case 142:lcd_print("RKB     ");break;
		case 143:lcd_print("HBC     ");break;
		case 144:lcd_print("NHK1    ");break;
		case 146:lcd_print("OBC     ");break;
		case 148:lcd_print("TOKAI   ");break;
		case 150:lcd_print("RCC     ");break;
		case 154:lcd_print("NHK2    ");break;
		case 156:lcd_print("SBS     ");break;
		case 157:lcd_print("KBC     ");break;
		case 158:lcd_print("RNIPPON ");break;
		case 160:lcd_print("STV     ");break;
		case 161:lcd_print("RNC     ");break;
		case 166:lcd_print("RSK     ");break;
		case 168:lcd_print("NHK2    ");break;
		case 170:lcd_print("CRT     ");break;
		case 173:lcd_print("FBC     ");break;
		default:break;
	}
}
void	lcd_kyokumei_sub(unsigned int	no)
{
unsigned int	sub;
	sub=no/9;
	lcd_xy(8,1);
	switch(sub){
		case 62:lcd_print("KOBE    ");break;
		case 63:lcd_print("SAPPORO ");break;
		case 66:lcd_print("TOKYO   ");break;
		case 68:lcd_print("FUKUOKA ");break;
		case 74:lcd_print("OSAKA   ");break;
		case 77:lcd_print("TOKYO   ");break;
		case 81:lcd_print("NAGOYA  ");break;
		case 83:lcd_print("SAPPORO ");break;
		case 84:lcd_print("KUMAMOTO");break;
		case 86:lcd_print("AKITA   ");break;
		case 90:lcd_print("TOKYO   ");break;
		case 91:lcd_print("NAGOYA  ");break;
		case 92:lcd_print("OSAKA   ");break;
		case 97:lcd_print("KUMAMOTO");break;
		case 99:lcd_print("SENDAI  ");break;
		case 106:lcd_print("TOKYO   ");break;
		case 110:lcd_print("KOCHI   ");break;
		case 112:lcd_print("OSAKA   ");break;
		case 113:lcd_print("FUKUOKA ");break;
		case 117:lcd_print("NAGOYA  ");break;
		case 121:lcd_print("SENDAI  ");break;
		case 126:lcd_print("TOKYO   ");break;
		case 127:lcd_print("KYOTO   ");break;
		case 131:lcd_print("OSAKA   ");break;
		case 132:lcd_print("KITAMI  ");break;
		case 136:lcd_print("KANAZAWA");break;
		case 138:lcd_print("TOKYO   ");break;
		case 140:lcd_print("SENDAI  ");break;
		case 141:lcd_print("TOKUSIMA");break;
		case 142:lcd_print("FUKUOKA ");break;
		case 143:lcd_print("SAPPORO ");break;
		case 144:lcd_print("MATSUE  ");break;
		case 146:lcd_print("OSAKA   ");break;
		case 148:lcd_print("NAGOYA  ");break;
		case 150:lcd_print("HIROSIMA");break;
		case 156:lcd_print("SHIZUOKA");break;
		case 157:lcd_print("FUKUOKA ");break;
		case 158:lcd_print("YOKOHAMA");break;
		case 160:lcd_print("SAPPORO ");break;
		case 161:lcd_print("KAGAWA  ");break;
		case 166:lcd_print("OKAYAMA ");break;
		case 170:lcd_print("TOCHIGI ");break;
		default:break;
	}
}

/*----------- Si47xx ----------*/
void wait_ns(u16 ns)
{
    u8 i;

    for ( i = 1; i <= ns / 32; i++ ) /* about */
    ;
}

void wait_us(u16 us)
{
    u16 j;

    j = 8 * us-1;
    CR000=j;
    TM00_Start();
    while(timer00_over != 1); 
	TM00_Stop();
	timer00_over=0;
}

void wait_ms(u16 ms)
{
    int i;

    for ( i = 0; i < ms; i++ ) {
        wait_us(1000);
    }
}

void si47xx_reset(void)
{

    RSTB = 0;

	PORT_ChangeP17Output(0);
	PORT_ChangeP15Output(1);	/* I2C */

    wait_us(100);
    RSTB = 1;
    wait_us(30);

	PORT_ChangeP17Input(1);
	PORT_ChangeP15Input(1);
}

void si47xx_lowRead(u8 number_bytes, u8 *data_in)
{

	i2c_radio_read(number_bytes,data_in);
	wait_i2c();
}

u8 si47xx_readStatus()
{
    u8 status;
   
    si47xx_lowRead(1, &status);

    return status;
}

void si47xx_waitForCTS()
{
    u16 i=1000;

    while (--i && !(si47xx_readStatus() & CTS))
    {
        wait_us(500);
    }
}

void si47xx_lowWrite(u8 number_bytes, u8 *data_out)
{
    i2c_radio_write(number_bytes,data_out);
    wait_i2c();
}


void si47xx_command(u8 cmd_size, u8 *cmd, u8 reply_size, u8 *reply)
{
    si47xx_waitForCTS();
    si47xx_lowWrite(cmd_size, cmd);
    si47xx_waitForCTS();
    if(reply_size)
    {
        si47xx_lowRead(reply_size, reply);
    }
}

u8 getIntStatus(void)
{
    u8  cmd[1] = {0};
    u8  rsp[1] = {0};

    cmd[0] = GET_INT_STATUS;
	si47xx_command(1, cmd, 1, rsp);
	return rsp[0];
}

void si47xx_set_property(u16 propNumber, u16 propValue)
{
    cmd[0] = SET_PROPERTY;
    cmd[1] = 0;
    cmd[2] = (u8)(propNumber >> 8);
	cmd[3] = (u8)(propNumber & 0x00FF);
    cmd[4] = (u8)(propValue >> 8);
    cmd[5] = (u8)(propValue & 0x00FF);
	si47xx_command(6, cmd, 0,  0);
}

void si47xx_getPartInformation(void)
{

	cmd[0] = GET_REV;

	si47xx_command(1, cmd, 9, rsp);

	partNumber = rsp[1];
	fwMajor  = (char)rsp[2];
	fwMinor  = (char)rsp[3];
	patchID  = (u16)(rsp[4] << 8) | (u16)rsp[5];
	cmpMajor = (char)rsp[6];
	cmpMinor = (char)rsp[7];
	chipRev  = (char)rsp[8]; 
	
}

void si47xxAMRX_powerup(void)
{
    if (Bit.PoweredUp) {
    } else {
		cmd[0] = POWER_UP;
        cmd[1] = (POWER_UP_IN_FUNC_AMRX | 0x10) ;
        cmd[2] = POWER_UP_IN_OPMODE_RX_ANALOG;
		si47xx_command(3, cmd, 8, rsp);
        wait_ms(POWERUP_TIME);
		Bit.PoweredUp = 1;
    }
}

void si47xxAMRX_powerdown(void)
{
    if(Bit.PoweredUp)
    {   
        Bit.PoweredUp = 0;
	    cmd[0] = POWER_DOWN;
		si47xx_command(1, cmd, 1, rsp);
    }
}
static void	si47xxAMRX_filer_cfg(void)
{
	si47xx_set_property(AM_CHANNEL_FILTER, AM_CHANNEL_FILTER_BW_3KHZ);
}

static void si47xxAMRX_hardware_cfg(void)
{
    si47xx_set_property(GPO_IEN, GPO_IEN_STCIEN_MASK);
}

static void si47xxAMRX_general_cfg(void)
{
    si47xx_set_property(AM_SEEK_TUNE_SNR_THRESHOLD, 5);
    si47xx_set_property(AM_SEEK_TUNE_RSSI_THRESHOLD, 25);
}

static void si47xxAMRX_regional_cfg(void)
{
    si47xx_set_property(AM_SEEK_FREQ_SPACING, 9);
    si47xx_set_property(AM_SEEK_BAND_BOTTOM, 522);
    si47xx_set_property(AM_SEEK_BAND_TOP, 1620);
}
void si47xxAMRX_configure(void)
{
    si47xxAMRX_hardware_cfg();
    si47xxAMRX_general_cfg(); 
    si47xxAMRX_regional_cfg();
	si47xxAMRX_filer_cfg();
}

void si47xxAMRX_initialize(void)
{
	Bit.PoweredUp = 0;
    si47xx_reset();
    si47xxAMRX_powerup();
    si47xxAMRX_configure();
}

void si47xxAMRX_set_volume(u8 volume)
{
    si47xx_set_property(RX_HARD_MUTE, 0);
    si47xx_set_property(RX_VOLUME, (u16)volume & RX_VOLUME_MASK);
}

void si47xxAMRX_mute(u8 mute)
{
    if(mute)
    	si47xx_set_property(RX_HARD_MUTE, 
                                RX_HARD_MUTE_RMUTE_MASK | RX_HARD_MUTE_LMUTE_MASK);
    else
    	si47xx_set_property(RX_HARD_MUTE, 0);
}

static void amTuneFreq(u16 frequency)
{
    cmd[0] = AM_TUNE_FREQ;
    cmd[1] = 0;
    cmd[2] = (u8)(frequency >> 8);
	cmd[3] = (u8)(frequency & 0x00FF);
    cmd[4] = (u8)0;
	cmd[5] = (u8)0;
	si47xx_command(6, cmd, 1, rsp);
}

static void amTuneStatus(u8 cancel, u8 intack)
{
    cmd[0] = AM_TUNE_STATUS;

	cmd[1] = 0;
    if(cancel) cmd[1] |= AM_TUNE_STATUS_IN_CANCEL;
	if(intack) cmd[1] |= AM_TUNE_STATUS_IN_INTACK;

	si47xx_command(2, cmd, 8, rsp);

    STC    = !!(rsp[0] & STCINT);
    BLTF   = !!(rsp[1] & AM_TUNE_STATUS_OUT_BTLF);
    AFCRL  = !!(rsp[1] & AM_TUNE_STATUS_OUT_AFCRL);
    Valid  = !!(rsp[1] & AM_TUNE_STATUS_OUT_VALID);
    Freq   = ((u16)rsp[2] << 8) | (u16)rsp[3];
    RSSI   = rsp[4];
    ASNR   = rsp[5];
    AntCap = ((u16)rsp[6] << 8) | (u16)rsp[7];   
}

static void amRsqStatus(void)
{
    cmd[0] = AM_RSQ_STATUS;

	cmd[1] = 1;

	si47xx_command(2, cmd, 6, rsp);

    RSSI   = rsp[4];
    ASNR   = rsp[5];
}

static void si47xxAMRX_tune(u16 frequency)
{
    amTuneFreq(frequency);
    while (!(getIntStatus() & STCINT));
    amTuneStatus(0, 1);
}
void	tune_sub(void)
{
    si47xxAMRX_tune(khz_no);
    if(RSSI>99){RSSI=99;}
    if(ASNR>99){ASNR=99;}
	lcd_xy(9,0);lcd_print("S");lcd_2keta10(RSSI);
	lcd_xy(13,0);lcd_print("N");lcd_2keta10(ASNR);
}

/*----------  KHZ    ----------*/
void	khz_inc_9(void)
{
	khz_no=khz_no+9;
	if(khz_no>FREQ_MAX){khz_no=FREQ_MIN;}
}

void	khz_dec_9(void)
{
	khz_no=khz_no-9;
	if(khz_no<FREQ_MIN){khz_no=FREQ_MAX;}
}
void	khz_inc_90(void)
{
	khz_no=khz_no+90;
	if(khz_no>FREQ_MAX){khz_no=FREQ_MIN;}
}

void	khz_dec_90(void)
{
	khz_no=khz_no-90;
	if(khz_no<FREQ_MIN){khz_no=FREQ_MAX;}
}

/*----------  A/D ----------*/
void	ad_input7(void)
{
    WTMK = 1;			/* INTWT Interrupt servicing disabled */
	AD_SelectADChannel(AD_INPUT_CHANNEL_7);
	AD_Read(&ad_lev_in);
    WTMK = 0;			/* INTWT Interrupt servicing enabled */
	
}
unsigned int	vol_yomi(void)
{
unsigned short	sa;

	ad_input7();
	vol_level=ad_lev_in;
	if(vol_level>vol_level_m){
		sa=vol_level-vol_level_m;
	}
	if(vol_level<=vol_level_m){
		sa=vol_level_m-vol_level;
	}
	if(sa>=4){
		vol_level_m=vol_level;
	}
	else{
		vol_level=vol_level_m;
	}
	vol_level=vol_level/FREQ_MANUAL_DIV;
	vol_level=vol_level*9+FREQ_MIN;
		
	
	return vol_level;
}

/*---------- PRESET ---------*/
unsigned char	preset_next(unsigned char no)
{
unsigned char i,ret;

	ret=no;
	no++;
	if(no>=PRESET_MAX){no=0;}
	for(i=no;i<PRESET_MAX;i++){
		if(preset[i]!=0){
			ret=i;
			return ret;
		}
	}
	for(i=0;i<no;i++){
		if(preset[i]!=0){
			ret=i;
			return ret;
		}
	}
	return ret;
}

unsigned char	preset_back(unsigned char no)
{
unsigned char i,ret;

	ret=no;
	if(no==0){no=PRESET_MAX-1;}else{no--;}
	for(i=no;i>=0;i--){
		if(preset[i]!=0){
			ret=i;
			return ret;
		}
	}
	for(i=PRESET_MAX-1;i>=0;i--){
		if(preset[i]!=0){
			ret=i;
			return ret;
		}
	}
	return ret;
}
/*---------------------------*/
void	preset_ch_init(void)
{
unsigned char i;

	for(i=0;i<127;i++){
		preset_ch[i]=0xff;
	}
}

void	preset_ch_set(void)
{
unsigned char	i;
unsigned int	khz;

	for(i=0;i<PRESET_MAX;i++){
		if(preset[i]!=0x00){
			khz=preset[i];
			preset_ch[(khz-FREQ_MIN)/9]=i;
		}
	}
}

/*---------- signal out ----------*/
void	head_out(void)
{
	set_count=4;
	TM51_Start();TUNE_OUT=1;
	while(set_count>0){}
	TM51_Stop();TUNE_OUT=0;
	set_count=4;
	TM51_Start();
	while(set_count>0){}
	TM51_Stop();
}
void	data1_out(void)
{
	set_count=1;
	TM51_Start();TUNE_OUT=1;
	while(set_count>0){}
	TM51_Stop();TUNE_OUT=0;
	set_count=3;
	TM51_Start();
	while(set_count>0){}
	TM51_Stop();
}
void	data0_out(void)
{
	set_count=1;
	TM51_Start();TUNE_OUT=1;
	while(set_count>0){}
	TM51_Stop();TUNE_OUT=0;
	set_count=1;
	TM51_Start();
	while(set_count>0){}
	TM51_Stop();
}
void	end_out(void)
{
	set_count=1;
	TM51_Start();TUNE_OUT=1;
	while(set_count>0){}
	TM51_Stop();TUNE_OUT=0;
	set_count=35;
	TM51_Start();
	while(set_count>0){}
	TM51_Stop();
}
void	data8_out(unsigned int data)
{
unsigned int d;
unsigned char i;

	for(i=0;i<8;i++){
		d=data & 0x01;
		if(d==0x01){data1_out();}
		else{data0_out();}
		data=data>>1;
	}
}
void	khz_data_out_am(unsigned int khz)
{
unsigned int n1;

	n1=khz/9;
	head_out();data8_out(n1);end_out();
}
/*---------- receive signal check ----------*/
void	receive_hantei10(void){

	receive_err=0;
	if((rec[1]>=57)&&(rec[1]<=69)){}else{receive_err=1;return;}
	
	receive_no=0;
	if((rec[2]>=26)&&(rec[2]<=39)){receive_no=receive_no | 0x01;}
	if((rec[3]>=26)&&(rec[3]<=39)){receive_no=receive_no | 0x02;}
	if((rec[4]>=26)&&(rec[4]<=39)){receive_no=receive_no | 0x04;}
	if((rec[5]>=26)&&(rec[5]<=39)){receive_no=receive_no | 0x08;}
	if((rec[6]>=26)&&(rec[6]<=39)){receive_no=receive_no | 0x10;}
	if((rec[7]>=26)&&(rec[7]<=39)){receive_no=receive_no | 0x20;}
	if((rec[8]>=26)&&(rec[8]<=39)){receive_no=receive_no | 0x40;}
	if((rec[9]>=26)&&(rec[9]<=39)){receive_no=receive_no | 0x80;}

}


/*
//---------------------------------------------------------------------------
//   MAIN
//---------------------------------------------------------------------------
*/
void  main( void )
{

unsigned int	sub;
unsigned char	sw1_key;
unsigned char	sw1_rep;
unsigned char	sw1_cnt;
unsigned char	sw2_key;
unsigned char	sw2_rep;
unsigned char	sw2_cnt;
unsigned char	sw3_key;
unsigned char	sw3_rep;
unsigned char	sw3_cnt;
unsigned char	sw3_off;
unsigned char	sw4_key;
unsigned char	sw4_rep;
unsigned char	sw4_cnt;

unsigned int	vol_count;


	sw1_key=0;sw1_rep=0;sw1_cnt=0;
	sw2_key=0;sw2_rep=0;sw2_cnt=0;
	sw3_key=0;sw3_rep=0;sw3_cnt=0;sw3_off=0;
	sw4_key=0;sw4_rep=0;sw4_cnt=0;

	timer00_over=0;

	Bit.PoweredUp=0;

	mode=1;preset_no=0;

	POWER_CONT=1;
	TUNE_OUT=0;
	
	rsq_count=RSQ_INIT;
	
	count10_flg=0;

	wait_time(50000);
	
	vol_count=VOL_COUNT_SET;
	
	AD_Start();
	
	lcd_init();
	lcd_clear();

	lcd_xy(0,1);
	lcd_print("NEW AM RECEIVER ");
	
	if((P1.1==0)&(P1.2==0)){
		eeprom_init();
	}
	else{
	
		if((P1.0==0)&(P1.1==1)&(P1.2==1)){
			bank_no=0;
			eeprom_write(0,0x0c,bank_no);wait_eeprom();
		}
		if((P1.0==1)&(P1.1==0)&(P1.2==1)){
			bank_no=1;
			eeprom_write(0,0x0c,bank_no);wait_eeprom();
		}
		if((P1.0==1)&(P1.1==1)&(P1.2==0)){
			bank_no=2;
			eeprom_write(0,0x0c,bank_no);wait_eeprom();
		}
		eeprom_check();
	}



	while(POWER_KEY==0){}

	kyoku_count=0;

    si47xxAMRX_initialize();
    si47xx_getPartInformation();
    
    lcd_xy(0,0);lcd_print("Si47  ");
    lcd_xy(4,0);lcd_2keta10(partNumber);
    lcd_xy(7,0);
    lcd_data(fwMajor);
    lcd_data(fwMinor);
    lcd_data(cmpMajor);
    lcd_data(cmpMinor);
    lcd_data(chipRev);
    
    lcd_xy(13,0);lcd_print("BK");
    lcd_su(bank_no);
    
    si47xxAMRX_set_volume(0);
    
	wait_time(50000);
	wait_time(50000);
	wait_time(50000);
	wait_time(50000);
	wait_time(50000);

	lcd_clear();
	lcd_ini_disp();

	preset_ch_init();
	preset_ch_set();

	TM50_Start();

	switch(mode){
		case 0:
				khz_no=preset[preset_no];
				lcd_xy(0,0);lcd_khz(khz_no);
				tune_sub();
				lcd_kyokumei_clr();
				lcd_mode(mode);
				khz_data_out_am(khz_no);
				kyoku_count=KYOKU_INIT;
				break;
		case 1:	if(tun_flg==1){
					khz_no=vol_yomi();
				}
				khz_no_m=khz_no;
				lcd_xy(0,0);lcd_khz(khz_no);
				tune_sub();
				lcd_kyokumei_clr();lcd_ch_disp();
				lcd_mode(mode);
				khz_data_out_am(khz_no);
				kyoku_count=KYOKU_INIT;
				break;
	}
	
	if(RSSI>=40){si47xxAMRX_set_volume(63);}
	
	while(POWER_KEY==0){}

	/* Start user code. Do not edit comment generated here */
	while (1) {
	
	/* key scan 1 */
		if (UP_KEY==0){
				sw1_key=0;
		}
		else{
			sw1_key=1;
			sw1_rep=0;
			sw1_cnt=0;
		}
		/*------------------*/
		if (DOWN_KEY==0){
				sw2_key=0;
		}
		else{
			sw2_key=1;
			sw2_rep=0;
			sw2_cnt=0;
		}
		/*------------------*/
		if (SELECT_KEY==0){
				sw3_key=0;
		}
		else{
			sw3_key=1;
			sw3_rep=0;
			/*sw3_cnt=0;*/
		}
		/*------------------*/
		if (POWER_KEY==0){
				sw4_key=0;
		}
		else{
			sw4_key=1;
			sw4_rep=0;
			sw4_cnt=0;
		}
		/*------------------*/

		wait_time(2000);
		
		/*-------------------*/

	/* key scan 2 */
		if ((UP_KEY==0)&&(sw1_key==0)){
			sw1_key=0;
		}
		else{
			sw1_key=1;
			sw1_rep=0;
			sw1_cnt=0;
		}
		/*------------------*/
		if ((DOWN_KEY==0)&&(sw2_key==0)){
			sw2_key=0;
		}
		else{
			sw2_key=1;
			sw2_rep=0;
			sw2_cnt=0;
		}
		/*------------------*/
		if ((SELECT_KEY==0)&&(sw3_key==0)){
			sw3_key=0;
		}
		else{
			sw3_key=1;
			sw3_rep=0;
			/*sw3_cnt=0;*/
			if(sw3_cnt>0){
				sw3_off=1;
			}
		}
		/*------------------*/
		if ((POWER_KEY==0)&&(sw4_key==0)){
			sw4_key=0;
		}
		else{
			sw4_key=1;
			sw4_rep=0;
			sw4_cnt=0;
		}
		/*------------------*/
		
	/* key action */
	
		if((sw1_key==0)&&(sw1_rep==0)){
			switch(mode){
				case 0:
						sw1_rep=1;
						si47xxAMRX_set_volume(0);
						preset_no=preset_next(preset_no);
						khz_no=preset[preset_no];
						lcd_xy(0,0);lcd_khz(khz_no);
						tune_sub();
						if(RSSI>=40){si47xxAMRX_set_volume(63);}
						sw1_cnt=0;
						lcd_kyokumei_clr();
						lcd_mode(mode);
						khz_data_out_am(khz_no);
						write_direct();
						kyoku_count=KYOKU_INIT;
						break;
				case 1:	
						sw1_rep=1;
						if(tun_flg==0){
							si47xxAMRX_set_volume(0);
							khz_inc_9();
							lcd_xy(0,0);lcd_khz(khz_no);
							tune_sub();
							if(RSSI>=40){si47xxAMRX_set_volume(63);}
							khz_no_m=khz_no;
							sw1_cnt=0;
							lcd_kyokumei_clr();
							lcd_ch_disp();
							lcd_mode(mode);
							khz_data_out_am(khz_no);
							write_direct();
							kyoku_count=KYOKU_INIT;
						}
						break;
				case 2:
						sw1_rep=1;
						preset_no_setmode++;
						if(preset_no_setmode>=PRESET_MAX){preset_no_setmode=1;}
						khz_no_setmode=preset[preset_no_setmode];
						lcd_xy(8,1);
						if(khz_no_setmode!=0){
							lcd_khz(khz_no_setmode);
							lcd_print("kHz");
						}
						else{
							lcd_print("=NONE=  ");
						}
						sw1_cnt=0;
						lcd_mode(mode);
						break;
			}
		}
		/*------------------*/
		if((sw1_key==0)&&(sw1_rep==1)){
			sw1_cnt++;
			if(sw1_cnt>=30){
				switch(mode){
					case 0:	break;
					case 1:
							if(tun_flg==0){
								si47xxAMRX_set_volume(0);
								khz_inc_90();
								lcd_xy(0,0);lcd_khz(khz_no);
								tune_sub();
								if(RSSI>=40){si47xxAMRX_set_volume(63);}
								sw1_cnt=0;
								khz_data_out_am(khz_no);
								write_direct();
								kyoku_count=KYOKU_INIT;
							}
							break;
					case 2:	break;
				}
			}
		}
		/*------------------*/
		if((sw2_key==0)&&(sw2_rep==0)){
			switch(mode){
				case 0:
						sw2_rep=1;
						si47xxAMRX_set_volume(0);
						preset_no=preset_back(preset_no);
						khz_no=preset[preset_no];
						lcd_xy(0,0);lcd_khz(khz_no);
						tune_sub();
						if(RSSI>=40){si47xxAMRX_set_volume(63);}
						sw2_cnt=0;
						lcd_kyokumei_clr();
						lcd_mode(mode);
						khz_data_out_am(khz_no);
						write_direct();
						kyoku_count=KYOKU_INIT;
						break;
				case 1:
						sw2_rep=1;
						if(tun_flg==0){
							si47xxAMRX_set_volume(0);
							khz_dec_9();
							lcd_xy(0,0);lcd_khz(khz_no);
							tune_sub();
							if(RSSI>=40){si47xxAMRX_set_volume(63);}
							khz_no_m=khz_no;
							sw2_cnt=0;
							lcd_kyokumei_clr();
							lcd_ch_disp();
							lcd_mode(mode);
							khz_data_out_am(khz_no);
							write_direct();
							kyoku_count=KYOKU_INIT;
						}
						break;
				case 2:
						sw2_rep=1;
						if(preset_no_setmode<=1){preset_no_setmode=PRESET_MAX-1;}
						else{preset_no_setmode--;}
						khz_no_setmode=preset[preset_no_setmode];
						lcd_xy(8,1);
						if(khz_no_setmode!=0){
							lcd_khz(khz_no_setmode);
							lcd_print("kHz");
						}
						else{
							lcd_print("=NONE=  ");
						}
						sw2_cnt=0;
						lcd_mode(mode);
						break;
			}
		}
		/*--------------------*/
		if((sw2_key==0)&&(sw2_rep==1)){
			sw2_cnt++;
			if(sw2_cnt>=30){
				switch(mode){
					case 0:	break;
					case 1:
							if(tun_flg==0){
								si47xxAMRX_set_volume(0);
								khz_dec_90();
								lcd_xy(0,0);lcd_khz(khz_no);
								tune_sub();
								if(RSSI>=40){si47xxAMRX_set_volume(63);}
								sw2_cnt=0;
								khz_data_out_am(khz_no);
								write_direct();
								kyoku_count=KYOKU_INIT;
							}
							break;
					case 2:	break;
				}
			}
		}
		/*--------------------*/
		if((sw3_key==0)&&(sw3_rep==0)){
			sw3_rep=1;
		}
		/*--------------------*/
		if((sw3_key==0)&&(sw3_rep==1)){
			sw3_cnt++;
		}
		/*--------------------*/
		if((sw3_key==1)&&(sw3_off==1)){
			if(sw3_cnt>=40){
				/**/
				switch(mode){
					case 0:	break;
					case 1:
							mode=2;
							preset_no_setmode=preset_no;
							khz_no_setmode=preset[preset_no];
							lcd_xy(8,1);
							lcd_khz(khz_no_setmode);
							lcd_print("kHz");
							sw1_cnt=0;
							lcd_mode(mode);
							break;
					case 2:
							if(preset_no_setmode!=0){
								mode=1;
								preset_no=preset_no_setmode;
								preset[preset_no]=0;
								preset_ch[(khz_no-FREQ_MIN)/9]=0xff;
								if(tun_flg==1){
									khz_no=vol_yomi();
								}
								lcd_xy(0,0);lcd_khz(khz_no);
								tune_sub();
								if(RSSI>=40){si47xxAMRX_set_volume(63);}
								lcd_kyokumei_clr();
								lcd_ch_disp();
								lcd_mode(mode);
								khz_data_out_am(khz_no);
								write_direct();
								write_direct_set(preset_no);
								preset_no=preset_next(preset_no);
							}
							break;
				}
			}
			else{
				/*Z*/
				switch(mode){
					case 0:
							mode=1;
							if(tun_flg==1){
								khz_no=vol_yomi();
							}
							lcd_xy(0,0);lcd_khz(khz_no);
							tune_sub();
							if(RSSI>=40){si47xxAMRX_set_volume(63);}
							lcd_kyokumei_clr();
							lcd_ch_disp();
							lcd_mode(mode);
							khz_data_out_am(khz_no);
							write_direct();
							kyoku_count=KYOKU_INIT;
							break;
					case 1:
							mode=0;
							khz_no=preset[preset_no];
							lcd_xy(0,0);lcd_khz(khz_no);
							tune_sub();
							if(RSSI>=40){si47xxAMRX_set_volume(63);}
							lcd_kyokumei_clr();
							lcd_mode(mode);
							khz_data_out_am(khz_no);
							write_direct();
							kyoku_count=KYOKU_INIT;
							break;
					case 2:
							if(preset_no_setmode!=0){
								mode=0;
								preset_no=preset_no_setmode;
								preset[preset_no]=khz_no;
								preset_ch[(khz_no-FREQ_MIN)/9]=preset_no;
								lcd_xy(0,0);lcd_khz(khz_no);
								tune_sub();
								if(RSSI>=40){si47xxAMRX_set_volume(63);}
								lcd_kyokumei_clr();
								lcd_mode(mode);
								write_direct();
								write_direct_set(preset_no);
							}
							break;
				}
			}
			
			sw3_off=0;
			sw3_cnt=0;
		}
		/*--------------------*/
		if((sw4_key==0)&&(sw4_rep==0)){
			switch(mode){
				case 0:	sw4_rep=1;
						break;
				case 1:	sw4_rep=1;
						switch(tun_flg){
							case 0:	tun_flg=1;
									khz_no=vol_yomi();
									break;
							case 1:	tun_flg=0;
									break;
						}
						lcd_xy(0,0);lcd_khz(khz_no);
						tune_sub();
						if(RSSI>=40){si47xxAMRX_set_volume(63);}
						khz_no_m=khz_no;
						lcd_kyokumei_clr();lcd_ch_disp();
						lcd_mode(mode);
						khz_data_out_am(khz_no);
						kyoku_count=KYOKU_INIT;
						break;
				case 2:	sw4_rep=1;
						mode=1;
						lcd_xy(0,0);lcd_khz(khz_no);
						tune_sub();
						if(RSSI>=40){si47xxAMRX_set_volume(63);}
						khz_no_m=khz_no;
						lcd_kyokumei_clr();
						lcd_ch_disp();
						lcd_mode(mode);
						khz_data_out_am(khz_no);
						write_direct();
						kyoku_count=KYOKU_INIT;
						break;
			}
		}
		/*------------------*/
		if((sw4_key==0)&&(sw4_rep==1)){
			sw4_cnt++;
			if(sw4_cnt>=30){
				switch(mode){
					case 0:
					case 1:
							lcd_xy(0,1);
							lcd_print("                ");
							lcd_xy(0,0);
							lcd_print("=== POWER OFF===");
							P12.2=0;
							while(POWER_KEY==0){}
							break;
					case 2:	break;
				}
			}
		}
		/*--------------------*/
		if((mode==1)&(tun_flg==1)){
		/*----------*/
		if(vol_count>0){vol_count--;}
		if(vol_count==0){
			khz_no=vol_yomi();
			if(khz_no>FREQ_MAX){khz_no=FREQ_MAX;}
			if(khz_no_m!=khz_no){
				lcd_xy(0,0);lcd_khz(khz_no);
				tune_sub();
				if(RSSI>=40){si47xxAMRX_set_volume(63);}
				lcd_kyokumei_clr();lcd_ch_disp();
				lcd_mode(mode);
				khz_data_out_am(khz_no);
				khz_no_m=khz_no;
				kyoku_count=KYOKU_INIT;
			}
			vol_count=VOL_COUNT_SET;
		}
		}
		/*---------------------*/

		if(kyoku_count>0){
			kyoku_count--;
			if(kyoku_count==KYOKU_ON){
				switch(mode){
					case 0:
					case 1:	lcd_xy(8,1);
							lcd_kyokumei(khz_no);
							break;
				}
			}
			if(kyoku_count==KYOKU_NEXT){
				switch(mode){
					case 0:
					case 1:	lcd_xy(8,1);
							lcd_kyokumei_sub(khz_no);
							break;
				}
			}
			if(kyoku_count==KYOKU_ON_2){
				switch(mode){
					case 0:
					case 1:	lcd_xy(8,1);
							lcd_kyokumei(khz_no);
							break;
				}
			}
			if(kyoku_count==0){
				switch(mode){
					case 0:	
							break;
					case 1:
							lcd_ch_disp_sub();
							break;
				}
			}
		}
		

	/*-------------------------*/
		if(rsq_count>0){
			rsq_count--;
			if(rsq_count==0){
				rsq_count=RSQ_INIT;
				amRsqStatus();
    			if(RSSI>99){RSSI=99;}
    			if(ASNR>99){ASNR=99;}
				lcd_xy(9,0);lcd_print("S");lcd_2keta10(RSSI);
				lcd_xy(13,0);lcd_print("N");lcd_2keta10(ASNR);
				if(RSSI>=40){si47xxAMRX_set_volume(63);}
			}
		}
	/*------------------------*/
	
	/* receive data hantei */
	
		if(count10_flg==1){
			count10_flg=0;
			receive_hantei10();
			if(receive_err==0){
				switch(receive_no){
					case 0x20:	
								lcd_xy(0,1);
								lcd_print("                ");
								lcd_xy(0,0);
								lcd_print("=== POWER OFF===");
								P12.2=0;
								while(POWER_KEY==0){}
								break;
					case 0x40:	if(mode==1){
									mode=0;
									khz_no=preset[preset_no];
									lcd_xy(0,0);lcd_khz(khz_no);
									tune_sub();
									if(RSSI>=40){si47xxAMRX_set_volume(63);}
									lcd_kyokumei_clr();
									lcd_mode(mode);
									khz_data_out_am(khz_no);
									write_direct();
									kyoku_count=KYOKU_INIT;
								}
								else{
									if(mode==0){
										si47xxAMRX_set_volume(0);
										preset_no=preset_next(preset_no);
										khz_no=preset[preset_no];
										lcd_xy(0,0);lcd_khz(khz_no);
										tune_sub();
										if(RSSI>=40){si47xxAMRX_set_volume(63);}
										lcd_kyokumei_clr();
										lcd_mode(mode);
										khz_data_out_am(khz_no);
										write_direct();
										kyoku_count=KYOKU_INIT;
									}
								}
								break;
					case 0x41:	if(mode==1){
									mode=0;
									khz_no=preset[preset_no];
									lcd_xy(0,0);lcd_khz(khz_no);
									tune_sub();
									if(RSSI>=40){si47xxAMRX_set_volume(63);}
									lcd_kyokumei_clr();
									lcd_mode(mode);
									khz_data_out_am(khz_no);
									write_direct();
									kyoku_count=KYOKU_INIT;
								}
								else{
									if(mode==0){
										si47xxAMRX_set_volume(0);
										preset_no=preset_back(preset_no);
										khz_no=preset[preset_no];
										lcd_xy(0,0);lcd_khz(khz_no);
										tune_sub();
										if(RSSI>=40){si47xxAMRX_set_volume(63);}
										lcd_kyokumei_clr();
										lcd_mode(mode);
										khz_data_out_am(khz_no);
										write_direct();
										kyoku_count=KYOKU_INIT;
									}
								}
								break;
					case 0x50:	
								mode=0;
								khz_no=preset[preset_no];
								lcd_xy(0,0);lcd_khz(khz_no);
								tune_sub();
								if(RSSI>=40){si47xxAMRX_set_volume(63);}
								lcd_kyokumei_clr();
								lcd_mode(mode);
								khz_data_out_am(khz_no);
								write_direct();
								kyoku_count=KYOKU_INIT;
								break;
				}
			}
		}


	}
	/* End user code. Do not edit comment generated here */
}

/* Start adding user code. Do not edit comment generated here */
/* End user code adding. Do not edit comment generated here */


